import React, { Component } from 'react'
import validators from './validators'
import ValidatorsEnum from './enum'

const decorateValidator = (validator) => {
  switch (validator.name) {
    case ValidatorsEnum.PRESENCE:
      return { ...validator, validate: validators.presence }
    case ValidatorsEnum.NUMERIC:
      return { ...validator, validate: validators.numeric }
    case ValidatorsEnum.LENGTH:
      return { ...validator, validate: validators.length(validator.maxLength) }
    case ValidatorsEnum.EMAIL:
      return { ...validator, validate: validators.email }
    default:
      return validator
  }
}

const createValidatedComponent = (InsideComponent) => {
  return class extends Component {
    constructor(props) {
      super(props)

      this._validate = this._validate.bind(this)
      this._validateValue = this._validateValue.bind(this)
      this._validateEverything = this._validateEverything.bind(this)

      this.state = {
        validations: {},
        isValid: false
      }
    }

    _validateValue(validation, key, value, state) {
      const isDisabled = validation.disabled ? validation.disabled(key, value, state) : false

      if (!validation.validate || isDisabled) {
        return { isValid: true, validationName: validation.name }
      }

      const isValid = validation.validate(validation.data ? validation.data(value) : value, key, state)

      return { isValid, validationName: validation.name }
    }

    _validateEverything(state) {
      const validations = Object.keys(this.props.validators).reduce((accumulator, validator) => {
        const { isValid, failed } = this._validateKey(validator, state[validator], state)
        return {
          validations: {
            ...accumulator.validations,
            [validator]: { isValid, failed: failed }
          },
          isValid: isValid && accumulator.isValid
        }
      }, { isValid: true })

      this.setState(
        ...this.state,
        validations
      )

      return validations
    }

    _validateKey(key, value, state = {}) {
      let isValid = true

      const keyValidator = this.props.validators[key] || []
      const decoratedValidators = Array
        .from(keyValidator.length ? keyValidator : [keyValidator])
        .map(decorateValidator)

      if (keyValidator.length === 0) {
        console.warn(`No validator defined for ${key}`)
      }

      const failed = decoratedValidators
        .map((validation) => this._validateValue(validation, key, value, state))
        .filter(({ isValid }) => !isValid)
        .map(({ validationName }) => validationName)

      if (failed && failed.length > 0) {
        isValid = false
      }

      return { isValid, failed }
    }

    _validate(key, value, state) {
      if (!key) {
        return this._validateEverything(state)
      }

      const { isValid, failed } = this._validateKey(key, value, state)
      const validations = {
        ...this.state.validations,
        [key]: { isValid, failed: failed }
      }

      this.setState({ isValid, validations })

      return validations
    }

    render() {
      return <InsideComponent
        {...this.props}
        validations={this.state.validations}
        validate={this._validate}
        isValid={this.state.isValid} />
    }
  }
}

export { createValidatedComponent, ValidatorsEnum }
